from . import gtk
from . import torrent
from . import layout
k=gtk.k
k3=gtk.k3

mask=gtk.GdkModifierType.GDK_CONTROL_MASK
key=gtk.GDK_KEY_n

@gtk.CALLBACK4i
def eve(tree, keyval, keycode, state):
	if (state&mask)==mask:
		K=k.gdk_keyval_to_lower(keyval)#shiftlock+n is N and lock mod NOT n and mod
		if K==key:
			mod=gtk.c_void_p()
			it=gtk.GtkTreeIter()
			i=gtk.byref(it)
			sl=k.gtk_tree_view_get_selection(tree)
			k.gtk_tree_selection_get_selected(sl,gtk.byref(mod),i)
			p=k.gtk_tree_model_get_path(mod,i)
			ix=k.gtk_tree_path_get_indices ( p )
			ix=ix[0]
			k.gtk_tree_path_free(p)
			find(sl,mod,i,ix)
			return True
	return False

def ini(tree):
	global controller
	controller=k.gtk_event_controller_key_new()
	k3.g_signal_connect_data (controller,b"key-pressed",eve,tree,None,gtk.GConnectFlags.G_CONNECT_SWAPPED)
	k.gtk_widget_add_controller (tree, controller)

def unini(tree):
	k.gtk_widget_remove_controller(tree,controller)
	
def find(sl,mod,it,pos):
	i2=gtk.GtkTreeIter()
	it2=gtk.byref(i2)
	b=k.gtk_tree_model_iter_next(mod,it)
	while b:
		if is_unfinished(mod,it,it2,sl):
			return
		b=k.gtk_tree_model_iter_next(mod,it)
	if pos>0:
		k.gtk_tree_model_get_iter_first(mod,it)
		while True:
			if is_unfinished(mod,it,it2,sl):
				return
			pos-=1
			if pos==0:
				break
			k.gtk_tree_model_iter_next(mod,it)

def is_unfinished(mod,it,it2,sl):
	k.gtk_tree_model_sort_convert_iter_to_child_iter(mod,it2,it)
	p2=k.gtk_tree_model_get_path(layout.list,it2)
	ix=k.gtk_tree_path_get_indices ( p2 )
	ix=ix[0]
	k.gtk_tree_path_free(p2)
	s=torrent.torrents[ix].h.status()
	if s.progress<1:
		p=k.gtk_tree_model_get_path(mod,it)
		k.gtk_tree_selection_select_path(sl,p)
		k.gtk_tree_path_free(p)
		torrent.pos(ix)
		return True
	return False

def sets(e):
	show(e)
	control=k.gtk_event_controller_key_new()
	k3.g_signal_connect_data (control,b"key-pressed",reset,e,None,gtk.GConnectFlags.G_CONNECT_SWAPPED)
	k.gtk_widget_add_controller (e, control)

def show(e):
	if (mask&gtk.GdkModifierType.GDK_CONTROL_MASK)!=0:
		txt="Ctrl+"
	else:
		txt=""
	if (mask&gtk.GdkModifierType.GDK_ALT_MASK)!=0:
		txt+="Alt+"
	txt+=k.gdk_keyval_name(key).decode()
	bf=k.gtk_entry_get_buffer(e)
	k.gtk_entry_buffer_set_text(bf,txt.encode(),-1)

@gtk.CALLBACK4i
def reset(e, keyval, keycode, state):
	global mask,key
	mask=state&(gtk.GdkModifierType.GDK_CONTROL_MASK|gtk.GdkModifierType.GDK_ALT_MASK)
	key=keyval
	show(e)
	return True

def store(d):
	d['next_mask']=mask
	d['next_key']=key
def restore(d):
	global mask,key
	mask=d['next_mask']
	key=d['next_key']



from . import gtk
from . import torben
from . import listtor
from . import conmenu
from . import sets
from . import confs
from . import stats
from . import torrent
from . import overall
from . import next
k=gtk.k
k3=gtk.k3

from enum import IntEnum
class COLUMNS(IntEnum):
	NAME=0
	PATH=1
	UP=2
	DOWN=3
	RATIO=4
	n=5

colsdef=lambda:k.gtk_list_store_new(COLUMNS.n, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING)
list=colsdef()
sort=k.gtk_tree_model_sort_new_with_model(list)
k3.g_object_unref(list)

@gtk.CALLBACK
def header_click(ix):
	ix=gtk.c_int(ix if ix else 0)#this comes none/int
	n=gtk.c_int()
	t=gtk.c_int()
	k.gtk_tree_sortable_get_sort_column_id(sort,gtk.byref(n),gtk.byref(t))
	if n.value!=ix.value or t.value!=gtk.GtkSortType.GTK_SORT_ASCENDING:
		k.gtk_tree_sortable_set_sort_column_id(sort,ix,gtk.GtkSortType.GTK_SORT_ASCENDING)
	else:
		k.gtk_tree_sortable_set_sort_column_id(sort,ix,gtk.GtkSortType.GTK_SORT_DESCENDING)
def columns_add(tree,n,i):
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(n, renderer, b"text", i, None)
	k3.g_signal_connect_data(column,b"clicked",header_click,i,None,gtk.GConnectFlags.G_CONNECT_SWAPPED)
	k.gtk_tree_view_column_set_clickable(column,True)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree, column)
	#if(w>0):
	#	k.gtk_tree_view_column_set_fixed_width(column,w)
def columns(tree):#,w #gtk_window_remembered_size is forcing *height=priv->height
	#w=int(w/COLUMNS.n)      #to shrink window size, it is required to move them manually, then resize
	columns_add(tree,b"Name",COLUMNS.NAME)
	columns_add(tree,b"Path",COLUMNS.PATH)
	columns_add(tree,b"Uploaded",COLUMNS.UP)
	columns_add(tree,b"Downloaded",COLUMNS.DOWN)
	columns_add(tree,b"Ratio",COLUMNS.RATIO)

@gtk.CALLBACK
def add(window):
	b=k.gtk_entry_get_buffer(entry_tor)
	t=k.gtk_entry_buffer_get_text(b)
	tx=t.decode()
	if torrent.open_tor(tx,0,window):
		tex=torben.name(tx)
		i=gtk.GtkTreeIter()
		ip=gtk.byref(i)
		if k.gtk_tree_model_iter_n_children(list,None)==0:
			next.ini(treeV)
		k.gtk_list_store_prepend(list,ip)
		gtk.gtk_list_store_set5(list, ip, COLUMNS.NAME, tex, COLUMNS.PATH, t,
			COLUMNS.UP,b"0",COLUMNS.DOWN,b"0",COLUMNS.RATIO,b"0")
		listtor.write(list)
def layout(window):
	bx=k.gtk_box_new(gtk.GtkOrientation.GTK_ORIENTATION_HORIZONTAL,0)
	global entry_tor
	entry_tor=k.gtk_entry_new()
	k.gtk_widget_set_hexpand(entry_tor,True)
	k3.g_signal_connect_data (entry_tor, b"activate", add, window, None, gtk.GConnectFlags.G_CONNECT_SWAPPED)
	#
	k.gtk_box_append(bx,entry_tor)
	b=k.gtk_button_new_with_label(b"+")
	k.gtk_box_append(bx,b)
	k3.g_signal_connect_data (b, b"clicked", add, window, None, gtk.GConnectFlags.G_CONNECT_SWAPPED)
	#
	b=k.gtk_button_new_with_label(chr(0x2699).encode())
	k3.g_signal_connect_data (b, b"clicked", sets.sets, window, None, gtk.GConnectFlags.G_CONNECT_SWAPPED)
	k.gtk_box_append(bx,b)
	#
	global treeV
	treeV=k.gtk_tree_view_new_with_model(sort)
	k3.g_object_unref(sort)
	columns(treeV) #,confs.width
	listtor.read(list,window)
	if k.gtk_tree_model_iter_n_children(list,None)>0:
		next.ini(treeV)
	#
	scroll = k.gtk_scrolled_window_new ()
	k.gtk_widget_set_vexpand(scroll,True)
	k.gtk_scrolled_window_set_child (scroll,treeV)
	#
	lst=overall.ini(colsdef())
	tree=k.gtk_tree_view_new_with_model(lst)
	k3.g_object_unref(lst)
	columns(tree) #,confs.width
	#
	box=k.gtk_box_new(gtk.GtkOrientation.GTK_ORIENTATION_VERTICAL,0)
	k.gtk_box_append(box,bx)
	k.gtk_box_append(box,scroll)
	k.gtk_box_append(box,tree)
	stats.ini(box)
	conmenu.ini(box,treeV)
	#
	k.gtk_window_set_child(window,box)


import libtorrent as lt

from . import gtk
from . import stats
from . import sets
from . import layout
from . import log
from . import ratio
k=gtk.k
k2=gtk.k2

ses = lt.session()
ses.listen_on(6881, 6891)
torrents=[]
timer=0

class tor():
	def __init__(self,h,u):
		self.h=h
		self.u=u

d_m='downloading metadata'.encode()
ckf='checking fastresume'.encode()
ck='checking'.encode()
state_str = ['queued'.encode(), \
	ck, d_m, 'downloading'.encode(), 'finished'.encode(), 'seeding'.encode(), \
	'allocating'.encode(), \
	ckf]
#queued and allocating are unused_enum_for_backwards_compatibility
#status().state.__int__() vs keys
#d meta
#chec fast,checking
#down,finish,sd
checki_states=[state_str.index(d_m),state_str.index(ckf),state_str.index(ck)]
def checki(s):
	for x in checki_states:
		if s.state==x:
			return True
	return False

@gtk.CALLBACK0i
def fresh():
	stats.show(h)
	return True

def pos(i):
	global timer
	if timer>0:
		k2.g_source_remove(timer)
	timer=k2.g_timeout_add(5000, fresh, None)
	global h
	h=torrents[i].h

def sel(tree,path):
	p2=k.gtk_tree_model_sort_convert_path_to_child_path(layout.sort,path)
	ix=k.gtk_tree_path_get_indices ( p2 )
	ix=ix[0]
	k.gtk_tree_path_free(p2)
	pos(ix)

def open_tor(path,u,w):
	try:
		with open(path+".fastresume") as f:
			b=f.read()
			c=eval(b)
			d=lt.bencode(c)
			td=lt.read_resume_data(d)
	except Exception:
		try:
			info = lt.torrent_info(path)
			pv=k.gtk_entry_buffer_get_text(sets.fold_bf)
			td={'ti': info, 'save_path': pv}
		except Exception:
			#the path is not existent
			print("Errors with: "+path)
			return False
	th=ses.add_torrent(td)#got no Name right after opening with libtor
	for x in torrents:
		if x.h==th:
			return False
	t=tor(th,u)
	torrents.insert(0,t)
	ratio.gain(w)
	log.addT(path)
	return True

def close():
	if timer>0:
		k2.g_source_remove(timer)
	i=gtk.GtkTreeIter()
	it=gtk.byref(i)
	mod=layout.list
	item_text=gtk.c_char_p()
	b=k.gtk_tree_model_get_iter_first(mod,it)
	j=0
	while b:
		gtk.gtk_tree_model_get (mod, it, layout.COLUMNS.PATH, gtk.byref(item_text))
		th=torrents[j].h
		with open(item_text.value.decode()+".fastresume", "w") as f:
			f.write(str(th.write_resume_data()))
		ses.remove_torrent(th)
		b=k.gtk_tree_model_iter_next(mod,it)
		j+=1

def remsel(i):
	global timer
	k2.g_source_remove(timer)
	timer=0
	ses.remove_torrent(h)
	del torrents[i]


from . import gtk
from . import torrent
from . import ratio
k=gtk.k

slot_bf=k.gtk_entry_buffer_new(b"-1",-1)
con_bf=k.gtk_entry_buffer_new(b"-1",-1)

def store(d):
	d['max_uploads']=getu()
	d['max_connections']=getc()
def restore(d):
	k.gtk_entry_buffer_set_text(slot_bf,str(d['max_uploads']).encode(),-1)
	k.gtk_entry_buffer_set_text(con_bf,str(d['max_connections']).encode(),-1)
	set()

def getu():
	return int(k.gtk_entry_buffer_get_text(slot_bf))
def getc():
	return int(k.gtk_entry_buffer_get_text(con_bf))
def set():
	torrent.ses.set_max_uploads(getu())
	torrent.ses.set_max_connections(getc())

def ini():
	grid = k.gtk_grid_new ()
	k.gtk_grid_attach(grid,ratio.text(b"Max uploads (-1 infinite)"),0,0,1,1)
	k.gtk_grid_attach(grid,ratio.edit(slot_bf),1,0,1,1)
	k.gtk_grid_attach(grid,ratio.text(b"Max connections (-1 infinite)"),0,1,1,1)
	k.gtk_grid_attach(grid,ratio.edit(con_bf),1,1,1,1)
	#
	fr=k.gtk_frame_new(b"Maximum limits")
	k.gtk_frame_set_child(fr,grid)
	return fr




import os.path
import appdirs
import pathlib

projname='torra'

def get_root_dir():
	return pathlib.Path(appdirs.user_config_dir(projname))
def get_root_file(f):
	return os.path.join(get_root_dir(),f)

import json

from . import gtk
from . import sets
from . import ratio
from . import next
from . import log
from . import cons
k=gtk.k

configs_filename = get_root_file('configs')

def write_opt(window):
	dict={}
	width=gtk.c_int()
	height=gtk.c_int()
	k.gtk_window_get_default_size (window, gtk.byref(width), gtk.byref(height))
	dict['max']=k.gtk_window_is_maximized(window)
	dict['width']=width.value
	dict['height']=height.value
	dict['min']=k.gdk_toplevel_get_state(k.gtk_native_get_surface(window))&gtk.GdkToplevelState.GDK_TOPLEVEL_STATE_MINIMIZED
	#
	dict['download_folder']=k.gtk_entry_buffer_get_text(sets.fold_bf).decode()
	ratio.store(dict)
	next.store(dict)
	log.store(dict)
	cons.store(dict)
	with open(configs_filename, "w") as write_file:
		json.dump(dict, write_file)

def read_opt(window):
	#global width#used at columns
	try:
		with open(configs_filename) as f:
			dict=json.load(f)
			k.gtk_window_set_default_size(window,dict['width'],dict['height'])  #to comeback here from maximize
			k.gtk_test_widget_wait_for_draw(window)  #widget_get_width and maximize without this? no
			if dict['max']:
				k.gtk_window_maximize(window)

			#width=-2 if dict['min'] else -1
			if dict['min']:
				k.gtk_window_minimize(window)

			a=dict['download_folder'].encode()
			k.gtk_entry_buffer_set_text(sets.fold_bf,a,-1)
			ratio.restore(dict)
			next.restore(dict)
			log.restore(dict)
			cons.restore(dict)
	except Exception:
		pass
	#	width=0



"""
torrent client
"""

import sys
import os

from . import confs
from . import gtk
from . import layout
from . import sets
from . import torrent
from . import overall
from . import listtor
from . import ratio
k=gtk.k
k3=gtk.k3
k4=gtk.k4

@gtk.CALLBACKi
def closing(window):
	listtor.write(layout.list)
	confs.write_opt(window)
	torrent.close()
	overall.close()
	ratio.freeint()
	return False

@gtk.CALLBACK
def activate(app):
	window = k.gtk_application_window_new (app)
	k.gtk_window_set_title (window, b"Torrent")
	k3.g_signal_connect_data (window, b"close-request", closing, None, None, 0)
	k.gtk_widget_show (window)
	confs.read_opt(window)

	#wd=confs.width
	#if wd<0:
	#	confs.width=k.gtk_widget_get_width(window)
	#	if wd==-2:
	#		k.gtk_window_minimize(window)

	layout.layout(window)

def main():
	if len(sys.argv)>1:
		if sys.argv[1]=="--remove-config":
			cleanup()
			return
		if sys.argv[1]=="--debug":
			sys.stdout.write("ENTRY_DEBUG marker\n")
			sys.stdout.flush()
		else:
			fname=sys.argv[1]
			if len(fname)==1:
				if len(sys.argv)>2:
					if fname[0]=='f':
						fname=sys.argv[2]
			listtor.config_filename=fname
	os.makedirs(confs.get_root_dir(),exist_ok=True)
	a=k.gtk_application_new(None,0)
	actv=b"activate"
	k3.g_signal_connect_data (a, actv, activate, None, None, 0)
	r=k4.g_application_run (a,0,None)
	k3.g_object_unref(sets.fold_bf)
	k3.g_object_unref(a)
	exit(r)

def cleanup_f(f):
	if os.path.isfile(f):
		print(f)
		return f
	return None
def cleanup():
	c=confs.get_root_dir()
	if os.path.isdir(c):
		print("Would remove(dirs only if empty):")
		f1=cleanup_f(confs.configs_filename)
		f2=cleanup_f(listtor.config_filename)
		print(c)
		base=os.path.dirname(c)
		bname=os.path.basename(base)
		if bname[0]=='.': #.config, on another system can be a premade
			print(base)
		elif bname==confs.projname: #if is torra/torra is ok
			print(base)
		else:
			base=None
		print("yes ?");
		str = ""
		while True:
			x = sys.stdin.read(1) # reads one byte at a time, similar to getchar()
			if x == '\n':
				break
			str += x
		if str=="yes":
			r=" removed"
			if f1:
				os.remove(f1)
				print(f1+r)
			if f2:
				os.remove(f2)
				print(f2+r)
			if len(os.listdir(path=c))==0:
				os.rmdir(c) #OSError if not empty
				print(c.__str__()+r)
				if base:
					if len(os.listdir(path=base))==0:
						os.rmdir(base)
						print(base.__str__()+r)
					else:
						print(base.__str__()+" is not empty.")
			else:
				print(c.__str__()+" is not empty.")
		else:
			print("expecting \"yes\"")

if __name__ == "__main__":
	main()




from enum import IntEnum,auto
class GdkModifierType(IntEnum):
	GDK_CONTROL_MASK = 1 << 2
	GDK_ALT_MASK = 1 << 3
class GdkToplevelState(IntEnum):
	GDK_TOPLEVEL_STATE_MINIMIZED = 1 << 0
class GtkAlign(IntEnum):
	GTK_ALIGN_START = auto()
class GConnectFlags(IntEnum):
	G_CONNECT_SWAPPED = 1 << 1
class GtkDialogFlags(IntEnum):
	GTK_DIALOG_MODAL = 1 << 0
	GTK_DIALOG_DESTROY_WITH_PARENT = 1 << 1
class GtkOrientation(IntEnum):
	GTK_ORIENTATION_HORIZONTAL=0
	GTK_ORIENTATION_VERTICAL=1
class GtkResponseType(IntEnum):
	GTK_RESPONSE_NONE = -1
class GtkSortType(IntEnum):
	GTK_SORT_ASCENDING=0
	GTK_SORT_DESCENDING=1
class GtkWrapMode(IntEnum):
	GTK_WRAP_WORD_CHAR=3

G_TYPE_FUNDAMENTAL_SHIFT = 2
G_TYPE_MAKE_FUNDAMENTAL=lambda x: x << G_TYPE_FUNDAMENTAL_SHIFT
G_TYPE_STRING=G_TYPE_MAKE_FUNDAMENTAL (16)
G_TYPE_INT=G_TYPE_MAKE_FUNDAMENTAL (6)
GDK_KEY_n=0x06e

import os
from ctypes import *

CALLBACK0i = CFUNCTYPE(c_int)
CALLBACK = CFUNCTYPE(None,c_void_p)
CALLBACKi = CFUNCTYPE(c_int,c_void_p)
CALLBACK2 = CFUNCTYPE(None,c_void_p,c_void_p)
CALLBACK3 = CFUNCTYPE(None,c_void_p,c_int,c_void_p)
CALLBACK4i = CFUNCTYPE(c_int,c_void_p,c_int,c_int,c_int)

class GtkTreeIter(Structure):
	_fields_=[("stamp",c_int),("user_data",c_void_p),("user_data2",c_void_p),("user_data3",c_void_p)]

#path_to_deps = "/usr/local/lib/arm-linux-gnueabihf"
#os.environ['PATH'] = path_to_deps + os.pathsep + os.environ['PATH']
try:
	k =cdll.LoadLibrary("libgtk-4.so.1")
	k2=k
	k3=k
	k4=k
except Exception:
	k =cdll.LoadLibrary("libgtk-4-1.dll")
	k2=cdll.LoadLibrary("libglib-2.0-0.dll")
	k3=cdll.LoadLibrary("libgobject-2.0-0.dll")
	k4=cdll.LoadLibrary("libgio-2.0-0.dll")

#By default functions are assumed to return the C int type. Other return types can be specified by setting the restype attribute of the function object.
#argtypes no default. c_void_p is python int. pointers must be announced on ARM.
#variadics are troubles

def gtk_tree_model_get(a,b,i1,p1):
	k.gtk_tree_model_get.argtypes=[c_void_p,c_void_p,c_int,c_void_p,c_int]
	k.gtk_tree_model_get(a,b,i1,p1,-1)
def gtk_tree_model_get2(a,b,i1,p1,i2,p2):
	k.gtk_tree_model_get.argtypes=[c_void_p,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int]
	k.gtk_tree_model_get(a,b,i1,p1,i2,p2,-1)
def gtk_list_store_set2(a,b,i1,p1,i2,p2):
	k.gtk_list_store_set.argtypes = [c_void_p,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int]
	k.gtk_list_store_set(a,b,i1,p1,i2,p2,-1)
def gtk_list_store_set3(a,b,i1,p1,i2,p2,i3,p3):
	k.gtk_list_store_set.argtypes = [c_void_p,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int]
	k.gtk_list_store_set(a,b,i1,p1,i2,p2,i3,p3,-1)
def gtk_list_store_set5(a,b,i1,v1,i2,p2,i3,p3,i4,p4,i5,p5):
	k.gtk_list_store_set.argtypes = [c_void_p,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int,c_void_p,c_int]
	k.gtk_list_store_set(a,b,i1,v1,i2,p2,i3,p3,i4,p4,i5,p5,-1)

k2.g_free.argtypes = [c_void_p]
k2.g_source_remove.argtypes = [c_void_p]
k2.g_timeout_add.argtypes = [c_int,c_void_p,c_void_p]
k3.g_object_unref.argtypes = [c_void_p]
k3.g_signal_connect_data.argtypes = [c_void_p,c_void_p,c_void_p,c_void_p,c_void_p,c_int]
k4.g_application_run.argtypes = [c_void_p,c_int,c_void_p]
#
k.gdk_keyval_name.restype=c_char_p
k.gdk_toplevel_get_state.argtypes = [c_void_p]
#A
k.gtk_application_new.restype=c_void_p
k.gtk_application_new.argtypes=[c_void_p,c_int]
k.gtk_application_window_new.restype=c_void_p
k.gtk_application_window_new.argtypes = [c_void_p]
#BO
k.gtk_box_append.argtypes = [c_void_p,c_void_p]
k.gtk_box_new.restype=c_void_p
#BU
k.gtk_button_new_with_label.restype=c_void_p
k.gtk_button_new_with_label.argtypes = [c_void_p]
#C
k.gtk_cell_renderer_progress_new.restype=c_void_p
k.gtk_cell_renderer_text_new.restype=c_void_p
#D
k.gtk_dialog_get_content_area.restype=c_void_p
k.gtk_dialog_get_content_area.argtypes = [c_void_p]
k.gtk_dialog_new_with_buttons.restype=c_void_p
k.gtk_dialog_new_with_buttons.argtypes = [c_void_p,c_void_p,c_int,c_void_p,c_int,c_void_p]
#EN
k.gtk_entry_buffer_get_text.restype=c_char_p
k.gtk_entry_buffer_get_text.argtypes = [c_void_p]
k.gtk_entry_buffer_new.restype=c_void_p
k.gtk_entry_buffer_set_text.argtypes = [c_void_p,c_void_p,c_int]
k.gtk_entry_get_buffer.restype=c_void_p
k.gtk_entry_get_buffer.argtypes = [c_void_p]
k.gtk_entry_new.restype=c_void_p
k.gtk_entry_new_with_buffer.restype=c_void_p
k.gtk_entry_new_with_buffer.argtypes = [c_void_p]
#EV
k.gtk_event_controller_key_new.restype=c_void_p
#F
k.gtk_frame_new.restype=c_void_p
k.gtk_frame_new.argtypes = [c_void_p]
k.gtk_frame_set_child.argtypes = [c_void_p,c_void_p]
#G
k.gtk_grid_attach.argtypes = [c_void_p,c_void_p,c_int,c_int,c_int,c_int]
k.gtk_grid_new.restype=c_void_p
#LA
k.gtk_label_new.restype=c_void_p
k.gtk_label_new.argtypes = [c_void_p]
#LI
k.gtk_list_store_append.argtypes = [c_void_p,c_void_p]
k.gtk_list_store_new.restype=c_void_p
k.gtk_list_store_prepend.argtypes = [c_void_p,c_void_p]
k.gtk_list_store_remove.argtypes = [c_void_p,c_void_p]
#N
k.gtk_native_get_surface.restype=c_void_p
k.gtk_native_get_surface.argtypes = [c_void_p]
#SC
k.gtk_scrolled_window_new.restype=c_void_p
k.gtk_scrolled_window_set_child.argtypes = [c_void_p,c_void_p]
#TEB
k.gtk_text_buffer_set_text.argtypes = [c_void_p,c_void_p,c_int]
#TEV
k.gtk_text_view_get_buffer.restype=c_void_p
k.gtk_text_view_get_buffer.argtypes = [c_void_p]
k.gtk_text_view_new.restype=c_void_p
k.gtk_text_view_set_editable.argtypes=[c_void_p,c_int]
k.gtk_text_view_set_wrap_mode.argtypes=[c_void_p,c_int]
#TEW
k.gtk_test_widget_wait_for_draw.argtypes = [c_void_p]
#TM
k.gtk_tree_model_get_iter_first.argtypes = [c_void_p,c_void_p]
k.gtk_tree_model_get_path.restype=c_void_p
k.gtk_tree_model_get_path.argtypes = [c_void_p,c_void_p]
k.gtk_tree_model_iter_n_children.argtypes = [c_void_p,c_void_p]
k.gtk_tree_model_iter_next.argtypes = [c_void_p,c_void_p]
k.gtk_tree_model_sort_convert_iter_to_child_iter.argtypes = [c_void_p,c_void_p,c_void_p]
k.gtk_tree_model_sort_convert_path_to_child_path.restype=c_void_p
k.gtk_tree_model_sort_convert_path_to_child_path.argtypes = [c_void_p,c_void_p]
k.gtk_tree_model_sort_new_with_model.restype=c_void_p
k.gtk_tree_model_sort_new_with_model.argtypes = [c_void_p]
#TP
k.gtk_tree_path_free.argtypes = [c_void_p]
k.gtk_tree_path_get_indices.restype=POINTER(c_int)
k.gtk_tree_path_get_indices.argtypes = [c_void_p]
#TS
k.gtk_tree_selection_get_selected.argtypes = [c_void_p,c_void_p,c_void_p]
k.gtk_tree_selection_select_path.argtypes = [c_void_p,c_void_p]
k.gtk_tree_sortable_get_sort_column_id.argtypes=[c_void_p,c_void_p,c_void_p]
k.gtk_tree_sortable_set_sort_column_id.argtypes=[c_void_p,c_int,c_int]
#TV
k.gtk_tree_view_get_selection.restype=c_void_p
k.gtk_tree_view_get_selection.argtypes = [c_void_p]
k.gtk_tree_view_append_column.argtypes = [c_void_p,c_void_p]
#TVC
k.gtk_tree_view_column_new_with_attributes.restype=c_void_p
k.gtk_tree_view_column_new_with_attributes.argtypes = [c_void_p,c_void_p,c_void_p,c_int,c_void_p]
k.gtk_tree_view_column_set_clickable.argtypes=[c_void_p,c_int]
k.gtk_tree_view_column_set_expand.argtypes=[c_void_p,c_int]
k.gtk_tree_view_column_set_resizable.argtypes = [c_void_p,c_int]
#
k.gtk_tree_view_new_with_model.restype=c_void_p
k.gtk_tree_view_new_with_model.argtypes = [c_void_p]
k.gtk_tree_view_set_activate_on_single_click.argtypes = [c_void_p,c_int]
#WID
k.gtk_widget_add_controller.argtypes = [c_void_p,c_void_p]
k.gtk_widget_get_height.argtypes=[c_void_p]
k.gtk_widget_get_root.restype=c_void_p
k.gtk_widget_get_root.argtypes=[c_void_p]
k.gtk_widget_get_width.argtypes=[c_void_p]
k.gtk_widget_hide.argtypes=[c_void_p]
k.gtk_widget_remove_controller.argtypes = [c_void_p,c_void_p]
k.gtk_widget_set_halign.argtypes = [c_void_p,c_int]
k.gtk_widget_set_hexpand.argtypes=[c_void_p,c_int]
k.gtk_widget_set_vexpand.argtypes=[c_void_p,c_int]
k.gtk_widget_show.argtypes=[c_void_p]
#WIN
k.gtk_window_close.argtypes=[c_void_p]
k.gtk_window_destroy.argtypes=[c_void_p]
k.gtk_window_get_default_size.argtypes=[c_void_p,c_void_p,c_void_p]
k.gtk_window_is_maximized.argtypes=[c_void_p]
k.gtk_window_maximize.argtypes=[c_void_p]
k.gtk_window_minimize.argtypes=[c_void_p]
k.gtk_window_set_child.argtypes = [c_void_p,c_void_p]
k.gtk_window_set_default_size.argtypes=[c_void_p,c_int,c_int]
k.gtk_window_set_title.argtypes=[c_void_p,c_void_p]



from . import gtk
from . import overall
from . import layout
from . import log
k=gtk.k
k2=gtk.k2

ratint_bf=k.gtk_entry_buffer_new(b"0",-1)
ratlim_bf=k.gtk_entry_buffer_new(b"2",-1)
timer=0

def text(a):
	tx=k.gtk_label_new(a)
	k.gtk_widget_set_halign(tx,gtk.GtkAlign.GTK_ALIGN_START)
	return tx
def edit(b):
	e=k.gtk_entry_new_with_buffer(b)
	k.gtk_widget_set_hexpand(e,True)
	return e

def ini():
	grid = k.gtk_grid_new ()
	k.gtk_grid_attach(grid,text(b"Interval time to verify in minutes (0=disable)"),0,0,1,1)
	k.gtk_grid_attach(grid,edit(ratint_bf),1,0,1,1)
	k.gtk_grid_attach(grid,text(b"Value"),0,1,1,1)
	k.gtk_grid_attach(grid,edit(ratlim_bf),1,1,1,1)
	#
	fr=k.gtk_frame_new(b"Close program when Ratio is greater than Value and torrents checking was done")
	k.gtk_frame_set_child(fr,grid)
	return fr

def freeint():
	global timer
	if timer>0:
		k2.g_source_remove(timer)
		timer=0
def setint(window):
	freeint()
	gain(window)

@gtk.CALLBACKi
def fresh(window):
	ratio=getratio()
	log.add(ratio)
	n=float(k.gtk_entry_buffer_get_text(ratlim_bf))
	if ratio>n:
		print("Ratio limit. Window close.")
		timer=0
		k.gtk_window_close(window)#it is freeint inside here
		return False
	return True
def gain(w):#when add tor and resets(including log file)
	global timer
	if timer==0:
		n=int(k.gtk_entry_buffer_get_text(ratint_bf))
		if n>0:
			timer=k2.g_timeout_add(n*60000,fresh,w)
	log.likenew()

def getratio():
	z=gtk.c_char_p()
	gtk.gtk_tree_model_get(overall.list,overall.it,layout.COLUMNS.RATIO,gtk.byref(z))
	r=float(z.value)
	k2.g_free(z)
	return r

def store(d):
	d['ratio_time']=int(k.gtk_entry_buffer_get_text(ratint_bf))
	d['ratio_limit']=float(k.gtk_entry_buffer_get_text(ratlim_bf))
def restore(d):
	a=str(d['ratio_time']).encode()
	k.gtk_entry_buffer_set_text(ratint_bf,a,-1)
	a=str(d['ratio_limit']).encode()
	k.gtk_entry_buffer_set_text(ratlim_bf,a,-1)



ver="1.0.40"
README="""Torrent client.\\
To remove configuration launch ```torra --remove-config```. There is also ```torra --debug```.\\
To specify another torrent list at start, launch with an argument, e.g. ```torra ~/torrents.txt```, or ```torra f ~/torrents.txt```\\
[Git Page](https://github.com/colin-i/tora)
"""

#this file will make setup.py after using github colin-i test/pyp/pypre
#setuptools.setup is looking at one argv parameter; to "build" and "install":
#python3 setup.py install

#libtorrent from pypi has bindings and library now, before was:
#	python-libtorrent-bin is at extra require now, but, if was at install requires:
#		ok, package python-libtorrent-bin is old. install with pip install --no-deps but add somehow appdirs
#			and python-libtorrent on ubuntu
#			if it's not old python-libtorrent at pip:
#				that+(libtorrent-rasterbar2.0 on ubuntu) can be a solution

pkname='torra'

from setuptools import setup
setup(name=pkname,
	version=ver,
	packages=[pkname],
	#opt
	python_requires='>=3',
	install_requires=["appdirs>=1.4.3"
		,"libtorrent"
		#python-libtorrent-bin it's not updated at pypi (old 3.9)
		#,'python-libtorrent-bin>=1.2.9' #;platform_system=="Linux" and platform_machine=="x86_64"'
		#,"python-apt"#is from 2012 0.7.8, missing DistUtilsExtra, sudo apt install python-apt is 2.2., verify with pip3 install python-apt
	],
	#extras_require={
	#	'bin': ['python-libtorrent-bin>=1.2.9']
	#	#,'apt': ['python-apt']
	#},
	description='Torrent client',
	long_description=README,
	long_description_content_type="text/markdown",
	url='https://github.com/colin-i/tora',
	author='cb',
	author_email='costin.botescu@gmail.com',
	license='MIT',
	entry_points = {
		'console_scripts': [pkname+'='+pkname+'.main:main']
	}
)



from . import gtk
from . import layout
k=gtk.k
k2=gtk.k2

def text(f,tree,data):
	mod=gtk.c_void_p()
	it=gtk.GtkTreeIter()
	i=gtk.byref(it)
	sl=k.gtk_tree_view_get_selection(tree)
	k.gtk_tree_selection_get_selected(sl,gtk.byref(mod),i)
	item_text=gtk.c_char_p()
	gtk.gtk_tree_model_get (mod, i, layout.COLUMNS.PATH, gtk.byref(item_text))
	f(item_text.value,data)
	k2.g_free(item_text)

def position__iter(f,tree):
	mod=gtk.c_void_p()
	it=gtk.GtkTreeIter()
	i=gtk.byref(it)
	sl=k.gtk_tree_view_get_selection(tree)
	k.gtk_tree_selection_get_selected(sl,gtk.byref(mod),i)
	path = k.gtk_tree_model_get_path (mod , i)
	p2=k.gtk_tree_model_sort_convert_path_to_child_path(mod,path)
	k.gtk_tree_path_free(path)
	ix=k.gtk_tree_path_get_indices ( p2 )
	ix=ix[0]
	k.gtk_tree_path_free(p2)
	#
	it2=gtk.GtkTreeIter()
	i2=gtk.byref(it2)
	k.gtk_tree_model_sort_convert_iter_to_child_iter(mod,i2,i)
	#
	f(ix,i2)



import libtorrent as lt

from . import gtk
from . import torrent
k=gtk.k
k3=gtk.k3

from enum import IntEnum
class COLUMNS(IntEnum):
	progress=0
	download_rate=1
	upload_rate=2
	num_peers=3
	state=4
	N=5
class COLUMNS2(IntEnum):
	list_peers=0
	num_complete=1
	num_incomplete=2
	next_announce=3
	last_upload=4
	N=5

#this is violation
#arr =  [24,64]
#arr_c = (gtk.c_int * 2)(*arr)
#k.gtk_list_store_newv.argtypes = [gtk.c_int, gtk.POINTER(gtk.c_int)]
#list=k.gtk_list_store_newv(gtk.c_int(2),arr_c)

#k.gtk_list_store_new is limited to 5 (five) entries, at six is violation

list=k.gtk_list_store_new(COLUMNS.N, gtk.G_TYPE_INT, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING)
i=gtk.GtkTreeIter()
ip=gtk.byref(i)
k.gtk_list_store_append(list,ip)

list2=k.gtk_list_store_new(COLUMNS2.N, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING, gtk.G_TYPE_STRING)
i2=gtk.GtkTreeIter()
ip2=gtk.byref(i2)
k.gtk_list_store_append(list2,ip2)

def ini(box):
	tree=k.gtk_tree_view_new_with_model(list)
	k3.g_object_unref(list)
	renderer = k.gtk_cell_renderer_progress_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Percentage", renderer, b"value", COLUMNS.progress, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Download kB/s", renderer, b"text", COLUMNS.download_rate, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Upload kB/s", renderer, b"text", COLUMNS.upload_rate, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Peers num", renderer, b"text", COLUMNS.num_peers, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"State", renderer, b"text", COLUMNS.state, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree, column)
	k.gtk_box_append(box,tree)
	#
	tree2=k.gtk_tree_view_new_with_model(list2)
	k3.g_object_unref(list2)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Peers list", renderer, b"text", COLUMNS2.list_peers, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree2, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Seeds", renderer, b"text", COLUMNS2.num_complete, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree2, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Leeches", renderer, b"text", COLUMNS2.num_incomplete, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree2, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Next Announce", renderer, b"text", COLUMNS2.next_announce, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree2, column)
	renderer = k.gtk_cell_renderer_text_new()
	column = k.gtk_tree_view_column_new_with_attributes(b"Last Upload", renderer, b"text", COLUMNS2.last_upload, None)
	k.gtk_tree_view_column_set_resizable(column,True)
	k.gtk_tree_view_column_set_expand(column,True)
	k.gtk_tree_view_append_column(tree2, column)
	k.gtk_box_append(box,tree2)

def show(h):
	s = h.status()
	if h.is_seed():
		val=gtk.c_int()
		gtk.gtk_tree_model_get(list,ip,COLUMNS.progress,gtk.byref(val))
		if val.value<100:
			gtk.gtk_list_store_set5(list, ip,
				COLUMNS.progress, 100,
				COLUMNS.download_rate, str(0).encode(),
				COLUMNS.upload_rate, str(s.upload_rate / 1000).encode(),
				COLUMNS.num_peers, str(s.num_peers).encode(),
				COLUMNS.state, torrent.state_str[s.state])
		else:
			gtk.gtk_list_store_set2(list, ip,
				COLUMNS.upload_rate, str(s.upload_rate / 1000).encode(),
				COLUMNS.num_peers, str(s.num_peers).encode())
	else:
		gtk.gtk_list_store_set5(list, ip,
			COLUMNS.progress, int(s.progress * 100),
			COLUMNS.download_rate, str(s.download_rate / 1000).encode(),
			COLUMNS.upload_rate, str(s.upload_rate / 1000).encode(),
			COLUMNS.num_peers, str(s.num_peers).encode(),
			COLUMNS.state, torrent.state_str[s.state])
	gtk.gtk_list_store_set5(list2, ip2,
		COLUMNS2.list_peers, str(s.list_peers).encode(),
		COLUMNS2.num_complete, str(s.num_complete).encode(),
		COLUMNS2.num_incomplete, str(s.num_incomplete).encode(),
		COLUMNS2.next_announce, str(s.next_announce).encode(),
		COLUMNS2.last_upload, str(s.last_upload).encode())
